/******************************/
/* Timetabling-specific stuff */
/******************************/

/* The constraint types: */

enum constraint_type
{
  CLASH,			/* E1 and E2 must not clash: blame E1 */
  CLASH_COPY,			/* .. but keep a copy for later info with E2 */
  BEFORE,			/* Event E must precede another: but blame E */
  EXCLUDE,			/* Event E must not be in slot S: blame E */
  PRESET			/* Event E must be in slot S: just make it so! */
};

/* The constraint type is used by ttdecode but not by brelaz */
struct constraint
{				/* used for timetable constraints of     */
  struct constraint *next;	/* various sorts: e1 != e2, e1 not in s, */
  int e1;			/* e1 must be in s, e1 must precede e2   */
  int e2;
  int s;
  double cpenalty;		/* what this constraint costs to break   */
  enum constraint_type ctype;	/* what type of constraint this is */
};

/* The following enum defines symbols for each type of entry in */
/* the timetable data file. The struct following it associates  */
/* keywords with each type, thus making it easy to add other    */
/* keywords just by adding extra lines to the struct - code     */
/* should NOT need changing.                                    */

enum entry_type
{
  END_LIST,			/* (marks end of struct array below)          */
  DATA_EVENTS,			/* Number of events is #1                     */
  DATA_SLOTS,			/* Number of slots (NOT per day) is #1        */
  DATA_DAY,			/* Total number of days is #1                 */
  DATA_SEATS,                   /* Number of seats available in any slot      */
  DATA_SPREAD,			/* Near-clashes if <= #1 apart                */
  DATA_CLASHTYPE,		/* Clash penalties are per-student? type=1..3 */
  PENALTY_CLASH,		/* Penalty for clash is #1                    */
  PENALTY_ORDER,		/* Penalty for order violation is #1          */
  PENALTY_EXCLUDE,		/* Penalty for event in bad slot for it is #1 */
  PENALTY_NEARCLASH,		/* Penalty for near-clash is #1               */
  CONSTRAIN_CLASH,		/* Event #1 and event #2 in different slots   */
  CONSTRAIN_BEFORE,		/* Event #1 must precede event #2             */
  CONSTRAIN_EXCLUDE,		/* Event #1 must not be in slot #2            */
  CONSTRAIN_PRESET		/* Event #1 must be in slot #2                */
};

#define CHUNK 10		/* The array where numbers get stashed first will    */
                                /* increase by this size whenever necessary. MUST BE */
                                /* AT LEAST 2.                                       */
#define ALL -1			/* MUST BE NEGATIVE: means "get all numbers on line" */
#define SLOTVAL(x) ((x)-'0')

/* struct tt_params defines one kind of line in a PGA tt file */

struct tt_params
{
  char *param;
  enum entry_type param_type;
  int args_wanted;
};

/* This lists all possible kinds of line (ie record) in a PGA tt file */
 
struct tt_params paramlist[]=
{
  /* string      type               numArgs to gather */
  {"events",     DATA_EVENTS,       1},
  {"slots",      DATA_SLOTS,        1},
  {"days",       DATA_DAY,          1},
  {"seats",      DATA_SEATS,        1},
  {"near-clash", DATA_SPREAD,       1},
  {"clash-type", DATA_CLASHTYPE,    1},
  {"p-clash",    PENALTY_CLASH,     1},
  {"p-order",    PENALTY_ORDER,     1},
  {"p-exclude",  PENALTY_EXCLUDE,   1},
  {"p-near",     PENALTY_NEARCLASH, 1},
  {"separate",   CONSTRAIN_CLASH,   ALL},
  {"order",      CONSTRAIN_BEFORE,  ALL},
  {"exclude",    CONSTRAIN_EXCLUDE, ALL},
  {"preset",     CONSTRAIN_PRESET,  2},
  {NULL,         END_LIST,          0}
};

/* This defines a node in the graph. The arcs array lists the arcs from   */
/* this node. The nc array is made of n_colours ints: nc[i] is the number */
/* of neighbours that are already coloured with colour i.                 */
/*   There will be an array of nodes (nodeData) so that nodeData[n]       */
/* describes node n. The chain pointers are used to build a queue showing */
/* the order in which to consider nodes for colouring.                    */
struct node
{
  int id;            /* ID number of this node */
  int colour;        /* Its colour (0..n_colours-1) or -1 if not yet done */
  int *nc;           /* Array of ints: how many neighbours have each colour */
  int *arcs;         /* Pointer to array of node IDs: ie the arcs of graph */
  int nArcs;         /* How many arcs (ie useful entries in array) */
  int maxArcs;       /* Length of array (maybe need to realloc?) */
  int colDegree;     /* How many colours used by neighbours */
  int nOccurs;       /* No. of instances of arcs from this node */
  int nStudents;     /* No of user-specified cliques/students containing it */
  struct node *next; /* Linked list: the queue of nodes to colour */
  struct node *prev; /* The queue is doubly linked */
};

